# -*- coding: UTF-8 -*-
#
###
# Application: wah!cade
# File:        joystick.py
# Description: Wah!Cade Joystick Class
# Copyright (c) 2007   zerodiv
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
from constants import *
import gtk
_ = gettext.gettext
pygame_imported = False
try:
    import pygame
    pygame_imported = True
except ImportError:
    print _('Warning: pygame module not found.  Joysticks not supported') 

class joystick:
    """pygame joystick class"""
    def __init__(self):
        """initialise"""
        self.state = {}
        self.devices = {}
        self.ctrls = {}
        if pygame_imported:
            pygame.init()
            pygame.joystick.init()
   
    def use_ini_controls(self, ctrlr_ini):
        """read controller ini file""" 
        if not pygame_imported:
            return
        for mw_keys in ctrlr_ini.ini_dict.itervalues():
            mw_keys = mw_keys.strip(' "').split(' | ')
            for mw_key in mw_keys:
                if mw_key[:3] == "JOY":
                    self.state[mw_key] = 0
                    (dev_num, control) = mw_key.split("_", 1)
                    dev_num = dev_num[3:] 
                    self.devices[int(dev_num) - 1] = None
        #
        num_joysticks = pygame.joystick.get_count()
        for dev_num in self.devices.iterkeys():
            if dev_num < num_joysticks:
                self.devices[dev_num] = pygame.joystick.Joystick(dev_num)
                self.devices[dev_num].init()
       
    def use_all_controls(self):
        """which joysticks"""
        if not pygame_imported:
            return
        num_joysticks = pygame.joystick.get_count()
        for dev_num in range(num_joysticks):
            self.devices[dev_num] = pygame.joystick.Joystick(dev_num)
            self.devices[dev_num].init()
            num_buttons = self.devices[dev_num].get_numbuttons()
            for button_num in range(num_buttons):
                mw_key = "JOY%s_BUTTON%s" % (dev_num + 1, button_num)
                ctrl_name = _('Joystick %s Button %s') % (dev_num + 1, button_num)
                self.state[mw_key] = 0
                self.ctrls[ctrl_name] = mw_key 
            if self.devices[dev_num].get_numaxes() > 2:
                mw_key = "JOY%s_" % (dev_num + 1)
                ctrl_name = _('Joystick %s ') % (dev_num + 1)
                self.state[mw_key + "UP"] = 0
                self.state[mw_key + "DOWN"] = 0
                self.state[mw_key + "LEFT"] = 0
                self.state[mw_key + "RIGHT"] = 0
                self.ctrls[ctrl_name + _('Up')] = mw_key + "UP"
                self.ctrls[ctrl_name + _('Down')] = mw_key + "DOWN"
                self.ctrls[ctrl_name + _('Left')] = mw_key + "LEFT"
                self.ctrls[ctrl_name + _('Right')] = mw_key + "RIGHT"
                                                             
    def poll(self, event_cb, initial_repeat_delay=40):
        """poll for joystick events"""
        if not pygame_imported:
            return 0
        repeat_delay = 4
        # give pygame a chance to do its magic with the joystick
        pygame.event.pump()
        # end polling if no joysticks are found
        num_joysticks = pygame.joystick.get_count()
        if num_joysticks == 0:
            return 0
        # get the focused window or return
        active_window = None
        windows = gtk.window_list_toplevels()
        for window in windows:
            if window.is_active():
                active_window = window
        if active_window == None: 
            return 1
        # check if any of our defined controlls were pressed
        mw_key_events = []
        for mw_key in self.state.iterkeys():
            (dev_num, joy_type) = (mw_key.split("_", 1))
            dev_num = int(dev_num[3:]) - 1
            # if binding is for an unfound joystick, continue to next binding
            if dev_num > num_joysticks - 1:
                continue
            if joy_type[:6] == "BUTTON":
                button_num = joy_type[6:]
                if self.devices[dev_num].get_button(int(button_num)):
                    self.state[mw_key] += 1
                    mw_key_events.append(mw_key)
                elif self.state[mw_key] > 0:
                    self.state[mw_key] = 0
                    mw_key_events.append(mw_key)
            elif joy_type in ["LEFT", "RIGHT", "UP", "DOWN"]:
                if joy_type in ["LEFT", "RIGHT"]:
                    axis_num = 0
                else:
                    axis_num = 1
                axis_value = self.devices[dev_num].get_axis(axis_num)
                if joy_type in ["UP", "LEFT"]:
                    if axis_value < -0.5:
                        self.state[mw_key] += 1
                        mw_key_events.append(mw_key)
                    elif self.state[mw_key] > 0:
                        self.state[mw_key] = 0
                        mw_key_events.append(mw_key)
                else:
                    if axis_value > 0.5:
                        self.state[mw_key] += 1
                        mw_key_events.append(mw_key)
                    elif self.state[mw_key] > 0:
                        self.state[mw_key] = 0
                        mw_key_events.append(mw_key)
        # send fake key-press events
        for mw_key in mw_key_events:
            if self.state[mw_key] == 0:
                e = gtk.gdk.Event(gtk.gdk.KEY_RELEASE)
                if active_window.window:
                    e.window = active_window.window
                    event_cb(active_window, e, "JOYSTICK", mw_key)
            elif (self.state[mw_key] == 1 or 
              self.state[mw_key] == initial_repeat_delay or
              (self.state[mw_key] > initial_repeat_delay and 
              (self.state[mw_key] - initial_repeat_delay) % repeat_delay == 0)):
                e = gtk.gdk.Event(gtk.gdk.KEY_PRESS)
                if active_window.window:
                    e.window = active_window.window
                    event_cb(active_window, e, "JOYSTICK", mw_key)
        return 1

