# -*- coding: utf-8 -*-
#
###
# Application: wah!cade
# File:        win_scrsaver.py
# Description: screen saver window
# Copyright (c) 2008   Andy Balcombe <http://www.anti-particle.com>
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
import os
import sys
import time
import gc

#gtk
import pygtk
if sys.platform != 'win32':
    pygtk.require('2.0')
import gtk
import gtk.glade
import gobject
import pango

#project modules
from constants import *
from wc_common import WahCade
#import vidinfo
import filters
gst_media_imported = False
if sys.platform != 'win32':
    try:
        import gst_media
        gst_media_imported = True
    except:
        pass

class WinScrSaver(WahCade):
    """Wah!Cade Screen Saver Window"""
    def __init__(self, WinMain):
        #set main window
        self.WinMain = WinMain
        #build the window
        self.winScrSaver = gtk.Window()
        self.fixd = gtk.Fixed()
        #self.drwVideo = gtk.DrawingArea()
        self.imgArtwork1 = gtk.Image()
        self.imgArtwork2 = gtk.Image()
        self.imgArtwork3 = gtk.Image()
        self.imgArtwork4 = gtk.Image()
        self.imgArtwork5 = gtk.Image()
        self.imgArtwork6 = gtk.Image()
        self.imgArtwork7 = gtk.Image()
        self.imgArtwork8 = gtk.Image()
        self.imgArtwork9 = gtk.Image()
        self.imgArtwork10 = gtk.Image()
        self.lblGameDescription = gtk.Label()
        self.lblMP3Name = gtk.Label()
        self.fixd.show()
        self.winScrSaver.add(self.fixd)
        self.lblGameDescription.show()
        self.lblMP3Name.show()
        self._main_images = [
            self.imgArtwork1,
            self.imgArtwork2,
            self.imgArtwork3,
            self.imgArtwork4,
            self.imgArtwork5,
            self.imgArtwork6,
            self.imgArtwork7,
            self.imgArtwork8,
            self.imgArtwork9,
            self.imgArtwork10]
        for widget in self._main_images:
            self.fixd.add(widget)
        self.winScrSaver.set_decorated(False)
        self.winScrSaver.set_transient_for(self.WinMain.winMain)
        self.winScrSaver.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
        self.winScrSaver.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
        #video widget
        self.video_playing = False
        if gst_media_imported:
            self.drwVideo = gst_media.VideoWidget()
            #w, h = self.winScrSaver.get_size()
            #self.drwVideo.set_size_request(w, h)
            self.video_player = gst_media.GstVideo(self.drwVideo)
            self.video_player.set_volume(self.WinMain.wahcade_ini.getint('movie_volume'))
        else:
            self.drwVideo = gtk.Image()
            self.video_player = None
        #self.drwVideo.connect_after('realize', self.on_drwVideo_realize)
        self.drwVideo.show()
        self.fixd.put(self.drwVideo, 0, 0)
        self.fixd.add(self.lblGameDescription)
        self.fixd.add(self.lblMP3Name)
        #get keyboard & mouse events
        self.winScrSaver.add_events(
            gtk.gdk.POINTER_MOTION_MASK |
            gtk.gdk.SCROLL_MASK |
            gtk.gdk.BUTTON_RELEASE_MASK |
            gtk.gdk.KEY_PRESS |
            gtk.gdk.KEY_RELEASE)
        #self.drwVideo.add_events(
        #    gtk.gdk.POINTER_MOTION_MASK |
        #    gtk.gdk.SCROLL_MASK |
        #    gtk.gdk.BUTTON_RELEASE_MASK |
        #    gtk.gdk.KEY_PRESS |
        #    gtk.gdk.KEY_RELEASE)
        #self.drwVideo.connect('key-press-event', self.WinMain.on_winMain_key_press)
        self.winScrSaver.connect('key-press-event', self.WinMain.on_winMain_key_press)
        self.winScrSaver.connect('key-release-event', self.WinMain.on_winMain_key_press)
        self.winScrSaver.connect('scroll-event', self.WinMain.on_winMain_key_press)
        self.winScrSaver.connect('motion-notify-event', self.WinMain.on_winMain_key_press)
        self.winScrSaver.realize()
        self.drwVideo.set_property('visible', False)
        #hide mouse?
        if not self.WinMain.wahcade_ini.getint('show_cursor'):
            self.hide_mouse_cursor(self.winScrSaver)
        #interval
        self.slide_duration = 10
        self.scr_timer_id = None
        self.running = False
        self.intro_exit_movie_playing = False
        #currently selected screen saver image / movie
        self.game_idx = 0
        #fullscreen?
        if self.WinMain.config_opts.fullscreen and not self.WinMain.config_opts.windowed:
            self.winScrSaver.fullscreen()

    def start_scrsaver(self, saver_type):
        """start screen saver"""
        #start duration timer
        self.video_artwork_widget =  self._main_images[(self.WinMain.emu_ini.getint('movie_artwork_no') - 1)]        
        self.slide_duration = self.WinMain.wahcade_ini.getint('slide_duration') * 1000
        self.running = True
        self.intro_exit_movie_playing = False
        self.saver_type = saver_type
        self.winScrSaver.show()
        if saver_type in ['slideshow', 'movie']:
            self.scr_timer_id = gobject.timeout_add(self.slide_duration, self.on_slide_timer)
            #show first game
            self.on_slide_timer()
        elif saver_type == 'launch_scr':
            #start external screen saver
            os.system(self.WinMain.emu_ini.get('scr_file'))

    def stop_scrsaver(self):
        """stop screen saver"""
        if self.scr_timer_id:
            gobject.source_remove(self.scr_timer_id)
        if self.video_playing:
            self.video_playing = False
            self.video_player.stop()
        self.running = False
        self.winScrSaver.hide()
        
    def play_movie(self, movie_file, movie_finished_cb=None):
        """play given movie file once, then exit"""
        self.running = False
        self.intro_exit_movie_playing = True
        self.winScrSaver.show()
        #play new vid
        self.video_playing = True
        self.drwVideo.set_property('visible', True)
        self.movie_finished_cb = movie_finished_cb 
        self.video_player.play(movie_file, self.movie_finished)
        
    def movie_finished(self):
        """intro / exit movie finished"""
        self.intro_exit_movie_playing = False
        self.stop_scrsaver()
        #do callback if supplied
        if self.movie_finished_cb:
            self.movie_finished_cb()
        
    def on_slide_timer(self):
        """change the image(s)"""
        layout_path = os.path.dirname(self.WinMain.layout_file)
        if self.running:
            if self.saver_type == 'slideshow':
                #change images
                self.game_idx = self.WinMain.get_random_game_idx()
                game_info = filters.get_game_dict(self.WinMain.lsGames[self.game_idx])
                for i, img in enumerate(self._main_images):
                    img_path = self.WinMain.emu_ini.get('artwork_%s_image_path' % (i + 1))
                    if img.get_property('visible'):
                        #get image filename
                        img_filename = self.get_artwork_image(
                            img_path, 
                            layout_path,
                            game_info, 
                            self.WinMain.current_emu, 
                            (i + 1))
                        #display image
                        self.display_scaled_image(img, img_filename, self.WinMain.keep_aspect)
            elif self.saver_type == 'movie':
                #play a movie
                movie_found = False
                cnt = 0
                while not movie_found or cnt > 10000:
                    cnt += 1 #ensure we don't go into a infinite loop
                    self.game_idx = self.WinMain.get_random_game_idx()
                    game_info = filters.get_game_dict(self.WinMain.lsGames[self.game_idx])
                    #vid file
                    vid_filename = self.get_video_file(
                        self.WinMain.emu_ini.get('movie_path'),
                        game_info)
                    movie_found = os.path.isfile(vid_filename)
                    if movie_found:
                        #stop existing vid playing
                        if self.video_playing:
                            self.video_playing = False
                            self.video_player.stop()
                        #play new vid
                        self.video_playing = True
                        self.drwVideo.set_property('visible', True)
                        self.video_player.play(vid_filename)
                        """
                        #get video info
                        #vid_info = vidinfo.RiffInfo(open(vid_filename, 'rb'))
                        #self.video_width = vid_info.video[0].width
                        #self.video_height = vid_info.video[0].height
                        #first, get pixbuf in order to determine aspect ratio
                        img_path = self.WinMain.emu_ini.get('artwork_%s_image_path' % (
                            self.WinMain.emu_ini.getint('movie_artwork_no')))
                        img_filename = self.get_artwork_image(
                            img_path, 
                            layout_path,
                            game_info, 
                            self.WinMain.current_emu, 
                            self.WinMain.emu_ini.getint('movie_artwork_no'))
                        #calling this will set self.video_width and self.video_height
                        pb = self.get_scaled_pixbuf(self.video_artwork_widget, img_filename, True)
                        #stop existing vid playing
                        if self.video_playing:
                            self.video_playing = False
                            self.video_player.stop()
                        #resize video vidget
                        self.video_playing = True
                        img_w, img_h = self.video_artwork_widget.get_size_request()
                        xpos = (img_w - self.video_width) / 2
                        self.fixd.move(
                            self.drwVideo,
                            self.fixd.child_get_property(self.video_artwork_widget, 'x') + xpos,
                            self.fixd.child_get_property(self.drwVideo, 'y'))
                        self.drwVideo.set_size_request(self.video_width, self.video_height)
                        """
            #set desc
            self.lblGameDescription.set_text(self.WinMain.lsGames[self.game_idx][GL_GAME_NAME])
            #collect any memory "leaks"
            gc.collect()
        else:
            #stop, so hide window
            self.winScrSaver.hide()
        #done
        return self.running
